\name{simSearch}
\alias{simSearch}
\alias{simSearch,xcmsFragments-method}
\title{Unrestricted precursor Metlin Search}
\description{
  A method for searching \eqn{MS^2} data against the METLIN Database without a precursor restraint.
}
\usage{
simSearch(object, ppmfrag=20, percent=50, file, fullReport=FALSE, ...)
}
\arguments{
  \item{object}{An xcmsFragment object generated by xcmsRaw.collect}
  \item{ppmfrag}{Error on fragment masses in ppm}
  \item{percent}{Percentage thresthold to use for identification}
  \item{file}{Name of the output files}
  \item{fullReport}{Should a full report be generated}
  \item{...}{Arguments to plot.metlin()}
}
\details{
  This method searches the METLIN database for similar MS/MS spectra and ranks them on a fragment score and a neutral loss score. Both of these scores work on a \code{score_fun} method. 
  The search takes xcmsFragment objects and searches the database with an unrestricted precursor, thereby searching all spectra in the METLIN database. The percent variable can be used to remove spectra that are below the accepted percentage similarity. The percentage similarity is an independant variable for both the fragment search and the neutral loss search. 
  The method generates two files when the fullReport variable is set to TRUE. The default file shows the top 5 m/z's most frequently matched. This gives a guide as fragments and neutral losses which should be inspected with a formula calculator. The second file which is printed to a variable and or to the second file is a full report of the data. This report includes metabolite names from the METLIN database and gives both the fragment score and the neutral loss score thereby giving a confidence to the likelyhood of the possible molecule/family of molecules.
}
\value{
  A data frame with the following columns:

\item{m/z}{Precursor m/z of the Experimental spectra}
\item{rtmin}{Start of the rt window}
\item{rtmax}{End of the rt window}
\item{Experiment Collision Energy}{Experimental spectra Collision Energy}
\item{Fragment Score}{Score of the Fragments}
\item{Neutral Score}{Score of the Neutral loss}
\item{Common Neutral loss}{m/z of the most matching neutral loss}
\item{Common Fragment}{m/z of the most matching fragment}
\item{Compound Name}{Name of the compound from METLIN}
\item{Metlin Mass}{The mass as reported by METLIN}
\item{Collision Energy}{The collision energy of the metlin spectra}

}
\references{
  H. Paul Benton, D.M. Wong, S.A.Strauger, G. Siuzdak "XC\eqn{MS^2}:"
  Analytical Chemistry 2008
}
\author{H. Paul Benton, \email{hpbenton@scripps.edu}}
\keyword{methods}
